@@ -671,6 +671,7 @@ class Sf:
 
 
 
+
     async def object_permission_bulk_check(self, objects: list, permissions: list ):
         async_object_permission_check = async_wrap(self.object_permission_check)
         result_list = await asyncio.gather(*[async_object_permission_check(object, permission ) for object in objects for permission in permissions])
@@ -728,6 +729,72 @@ class Sf:
         
         return {'profiles_with_apex_page_access': profiles_with_apex_page_access, 'permission_sets_with_apex_page_access': permission_sets_with_apex_page_access}
 
+    # Retrieves the Page Layout by Profile by Record Type data for listed objects in the org
+    async def retrieve_profile_layout_record_type_matrix_by_objects(self, objects_to_retrieve_profile_layout_matrix = ['Account','Address_vod__c','Child_Account_vod__c'], discard_unqueriable_objects = False, return_pivot_table = False):
+        
+        objects_to_retrieve = [object[:-3] if object.endswith('__c') else object for object in objects_to_retrieve_profile_layout_matrix]
+        
+        # Objects in the Profile Layout table's TableEnumOrId column where the value is an object name instead of an object ID
+        
+        ENUM_OBJECTS = {'Account', 'AccountTeamMember', 'Asset', 'AuthorizationForm', 'AuthorizationFormConsent', 'BusinessBrand', 'Campaign', 'CampaignMember', 'Case',
+                            'CaseClose', 'CaseInteraction', 'CommunityMemberLayout', 'Contact', 'ContactPointAddress', 'ContactPointEmail', 'ContactPointPhone', 'ContentVersion', 'Contract', 'Customer', 'DelegatedAccount',
+                            'DuplicateRecordItem', 'DuplicateRecordSet', 'EmailMessage', 'Event', 'FeedItem', 'Global', 'Idea', 'Individual', 'Lead', 'Macro', 'ObjectTerritory2AssignmentRule',
+                            'Opportunity', 'OpportunityLineItem', 'Order', 'OrderItem', 'PersonAccount', 'Pricebook2', 'PricebookEntry', 'ProcessException', 'Product2',
+                            'ProfileSkill', 'ProfileSkillEndorsement', 'ProfileSkillUser', 'QuickText', 'Scorecard', 'ScorecardAssociation', 'ScorecardMetric', 'Seller', 'ServiceAppointmentGroup',
+                            'ServiceTerritoryRelationship', 'SignupRequest', 'SocialPersona', 'SocialPost', 'Solution', 'Task', 'Territory2', 'Territory2Model', 'Territory2Type', 'User',
+                            'UserAlt', 'UserProvAccount', 'UserProvisioningLog', 'UserProvisioningRequest', 'UserTerritory2Association', 'WorkProcedure', 'WorkProcedureStep', 'WorkTypeExtension'}
+        
+        # Objects that returns empty results if queried via the WHERE clause using Salesforce Tooling API.
+        # i.e. "Select LayoutId from ProfileLayout where TableEnumOrId = 'DelegatedAccount'" returns empty results.
+        UNQUERIABLE_ENUM_OBJECTS = {'DelegatedAccount','ProfileSkill','ProfileSkillEndorsement','ProfileSkillUser','ServiceTerritoryRelationship','WorkTypeExtension'}
+        
+        
+        
+        object_dataframe = pd.DataFrame(sf.tooling_query_all("Select DeveloperName from CustomObject"))
+        object_dataframe['ObjectID'] = object_dataframe.apply(lambda row: row['attributes']['url'].split('/')[-1] if row['attributes']['url'] else "", axis=1)
+        
+        profile_layout_queries = []
+        
+        for object_name in objects_to_retrieve:
+            if object_name in UNQUERIABLE_ENUM_OBJECTS and discard_unqueriable_objects == False:
+                raise Exception("Unqueriable object: " + object_name + ". Please use retrieve_profile_layout_record_type_matrix_all() instead or set discard_unqueriable_objects parameter to True")
+            elif object_name in ENUM_OBJECTS:
+                profile_layout_queries.append("Select LayoutId, ProfileId, Profile.Name, RecordTypeId, RecordType.Name, Layout.Name, TableEnumOrId from ProfileLayout where TableEnumOrId = '" + object_name + "'")
+            else:
+                object_id = object_dataframe[object_dataframe['DeveloperName'] == object_name]['ObjectID'].values[0]
+                profile_layout_queries.append(f"Select LayoutId, ProfileId, Profile.Name, RecordTypeId, RecordType.Name, Layout.Name, TableEnumOrId from ProfileLayout where TableEnumOrId = '{object_id}'")
+        
+        
+        async_tooling_query_all = async_wrap(self.tooling_query_all)
+        
+        
+        await_results = await asyncio.gather(*[async_tooling_query_all(query) for query in profile_layout_queries])
+        
+        await_result_dataframe = [transform_sf_result_set_rec(result) for result in await_results if len(result) > 0]
+        
+        account_page_layouts_bulk = pd.concat(await_result_dataframe)
+
+        account_page_layouts_bulk['ProfileName'] = account_page_layouts_bulk.apply(lambda row: row['ProfileLayout.Profile']['Name'] if row['ProfileLayout.Profile'] else "", axis=1)
+        account_page_layouts_bulk['RecordTypeName'] = account_page_layouts_bulk.apply(lambda row: row['ProfileLayout.RecordType']['Name'] if row['ProfileLayout.RecordType'] else "", axis=1)
+        account_page_layouts_bulk['PageLayout'] = account_page_layouts_bulk.apply(lambda row: row['ProfileLayout.Layout']['Name'] if row['ProfileLayout.Layout'] else "", axis=1)
+        account_page_layouts_bulk.drop(['ProfileLayout.Profile','ProfileLayout.RecordType','ProfileLayout.Layout'], axis=1, inplace=True)
+
+        object_dataframe['ObjectID'] = object_dataframe.apply(lambda row: row['attributes']['url'].split('/')[-1] if row['attributes']['url'] else "", axis=1)
+
+        account_page_layouts_bulk = account_page_layouts_bulk.merge(object_dataframe, left_on='ProfileLayout.TableEnumOrId', right_on='ObjectID', how='left').copy()
+        account_page_layouts_bulk['DeveloperName'] = account_page_layouts_bulk.apply(lambda row: row['DeveloperName'] if pd.notnull(row['DeveloperName']) else row['ProfileLayout.TableEnumOrId'], axis=1)
+        account_page_layouts_bulk.drop(['ObjectID','attributes'], axis=1, inplace=True)
+        
+        # Fill in the RecordTypeName colum with the Master Record Type Name
+        account_page_layouts_bulk['RecordTypeName'] = account_page_layouts_bulk['RecordTypeName'].apply(lambda row: row if row else 'Master')
+        # Filter out Deprecated Profiles
+        account_page_layouts_bulk = account_page_layouts_bulk[(~account_page_layouts_bulk['ProfileName'].isnull()) & (account_page_layouts_bulk['ProfileName'] != '') & (account_page_layouts_bulk['PageLayout'] != 'Veeva Vpro Unit Testing Layout')].copy()
+        
+        if return_pivot_table:
+            return account_page_layouts_bulk.pivot(index='ProfileName', columns=['DeveloperName','RecordTypeName'], values='PageLayout')
+        else:
+            return account_page_layouts_bulk
+
 
     
     ### ----------------------------------------------------------------------------------------------------
@@ -972,7 +1039,38 @@ class Sf:
         
         return {'permissionSetPermissions': permissionSetPermissions, 'profilePermissions': profilePermissions}
 
+    # Retrieves the Page Layout by Profile by Record Type data for all objects in the org
+    def retrieve_profile_layout_record_type_matrix_all(self, return_pivot_table = False):
+        account_page_layouts_bulk = transform_sf_result_set_rec(self.tooling_query_all("""select LayoutId, 
+                                                                                    ProfileId, Profile.Name, 
+                                                                                    RecordTypeId, RecordType.Name, 
+                                                                                    Layout.Name, TableEnumOrId from 
+                                                                                    ProfileLayout"""))
 
+        account_page_layouts_bulk['ProfileName'] = account_page_layouts_bulk.apply(lambda row: row['ProfileLayout.Profile']['Name'] if row['ProfileLayout.Profile'] else "", axis=1)
+        account_page_layouts_bulk['RecordTypeName'] = account_page_layouts_bulk.apply(lambda row: row['ProfileLayout.RecordType']['Name'] if row['ProfileLayout.RecordType'] else "", axis=1)
+        account_page_layouts_bulk['PageLayout'] = account_page_layouts_bulk.apply(lambda row: row['ProfileLayout.Layout']['Name'] if row['ProfileLayout.Layout'] else "", axis=1)
+        account_page_layouts_bulk.drop(['ProfileLayout.Profile','ProfileLayout.RecordType','ProfileLayout.Layout'], axis=1, inplace=True)
+
+
+        object_dataframe = pd.DataFrame(self.tooling_query_all("Select DeveloperName from CustomObject"))
+        object_dataframe['ObjectID'] = object_dataframe.apply(lambda row: row['attributes']['url'].split('/')[-1] if row['attributes']['url'] else "", axis=1)
+
+
+        account_page_layouts_bulk = account_page_layouts_bulk.merge(object_dataframe, left_on='ProfileLayout.TableEnumOrId', right_on='ObjectID', how='left').copy()
+        account_page_layouts_bulk['DeveloperName'] = account_page_layouts_bulk.apply(lambda row: row['DeveloperName'] if pd.notnull(row['DeveloperName']) else row['ProfileLayout.TableEnumOrId'], axis=1)
+        account_page_layouts_bulk.drop(['ObjectID','attributes'], axis=1, inplace=True)
+            
+        # Fill in the RecordTypeName colum with the Master Record Type Name
+        account_page_layouts_bulk['RecordTypeName'] = account_page_layouts_bulk['RecordTypeName'].apply(lambda row: row if row else 'Master')
+        # Filter out Deprecated Profiles
+        account_page_layouts_bulk = account_page_layouts_bulk[(~account_page_layouts_bulk['ProfileName'].isnull()) & (account_page_layouts_bulk['ProfileName'] != '') & (account_page_layouts_bulk['PageLayout'] != 'Veeva Vpro Unit Testing Layout')]
+        
+        if return_pivot_table:
+            return account_page_layouts_bulk.pivot(index='ProfileName', columns=['DeveloperName','RecordTypeName'], values='PageLayout')
+        else:
+            return account_page_layouts_bulk
+    
 
     def load_veeva_common(self):
         self.veeva_common = self.query("Select * From Veeva_Common_vod__c").to_dict()
